import { IObservable, Nub } from "jalhyd";

import { FormResultSection } from "../form-result-section";
import { FormDefSection } from "../form-def-section";
import { FormComputeSectionParametree } from "../form-compute-section-parametree";
import { FieldSet } from "../../fieldset";
import { FormulaireBase } from "./form-base";

export class FormulaireSectionParametree extends FormulaireBase {

    private _formSection: FormDefSection;

    constructor() {
        super();
        this._formSection = new FormDefSection(this);
        this._formResult = new FormResultSection(this);

        // remove obsolete observer set by super()
        this.removeObserver(this._formCompute);
        this._formCompute = new FormComputeSectionParametree(this, this._formSection, (this._formResult as FormResultSection));

        // default properties
        this._props["varCalc"] = "Hs";
    }

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._formSection.parseOptions(json);
    }

    public afterParseFieldset(fs: FieldSet) {
        this._formSection.afterParseFieldset(fs);
    }

    // interface Observer

    update(sender: IObservable, data: any) {

        // super.update(sender, data);
        if (sender instanceof Nub) {
            switch (data.action) {
                case "resultUpdated":
                    // forward Nub results update notification to FormCompute objects
                    this.notifyNubResultUpdated(sender);
                    break;
            }
        }

        // changement de propriété du FieldSet contenant le select de choix du type de section
        if (sender instanceof FieldSet && data.action === "propertyChange") {
            switch (sender.id) {
                case "fs_section":
                    this.replaceCurrentNub(sender.properties);
                    for (const fs of this.allFieldsets) {
                        fs.setNub(this._currentNub);
                        // treat the fieldset as new to re-subscribe to Nub properties change events
                        this.afterParseFieldset(fs);
                    }
                    this.reset();
                    break;

                case "fs_computed_var":
                    this.reset();
                    break;
            }
        }
    }
}
