import { Injectable } from "@angular/core";
import { MatSnackBar } from "@angular/material";

/**
 * Displays a notifications queue as consecutive snackbars
 */
@Injectable()
export class NotificationsService {

    /** FIFO queue for notifications */
    private notifications: any[];

    private isOpen: boolean;

    public constructor(
        private snackBar: MatSnackBar
    ) {
        this.notifications = [];
        this.isOpen = false;
    }

    /** Push a notification and display it as soon as possible */
    public notify(message: string, duration: number, action: string = "OK") {
        this.notifications.push({
            message: message,
            duration: duration,
            action: action
        });
        this.show();
    }

    /** Show all messages in the FIFO queue one after another */
    public show() {
        if (! this.isOpen) {
            // process next notification
            if (this.notifications.length > 0) {
                const notif = this.notifications.shift();
                this.isOpen = true;
                const ref = this.snackBar.open(notif.message, notif.action, {
                    duration: notif.duration
                });
                ref.afterDismissed().subscribe(() => {
                    this.isOpen = false;
                    this.show();
                });
            }
        }
    }
}
